{% load i18n wagtailadmin_tags %}

<li>
    {% icon name="doc-full" %}
    <a href="{% url 'wagtaildocs:index' %}">
        {% blocktrans trimmed count counter=total_docs with total_docs|intcomma as total %}
            {{ total }} Document <span class="w-sr-only">created in {{ site_name }}</span>
        {% plural %}
            {{ total }} Documents <span class="w-sr-only">created in {{ site_name }}</span>
        {% endblocktrans %}
    </a>
</li>
