# Generated by Django 1.9.1 on 2016-02-03 13:50
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("wagtailcore", "0024_alter_page_content_type_on_delete_behaviour"),
        ("wagtailcore", "0027_fix_collection_path_collation"),
    ]

    operations = []
