{% load i18n wagtailadmin_tags %}
{% if page_types_restricted %}
    {% trans "Choose" as choose_str %}
{% else %}
    {% trans "Choose a page" as choose_str %}
{% endif %}

{% querystring as search_query_params %}
{% include "wagtailadmin/shared/header.html" with title=choose_str subtitle=page_type_names|join:", " search_url="wagtailadmin_choose_page_search" query_parameters=search_query_params icon="doc-empty-inverse" search_disable_async=True %}

<div class="nice-padding">
    {% include 'wagtailadmin/chooser/_link_types.html' with current='internal' %}

    {% if page_types_restricted %}
        <p class="help-block help-warning">
            {% icon name='warning' %}
            {% blocktrans trimmed with type=page_type_names|join:", " count counter=page_type_names|length %}
                Only pages of type "{{ type }}" may be chosen for this field. Search results will exclude pages of other types.
            {% plural %}
                Only the following page types may be chosen for this field: {{ type }}. Search results will exclude pages of other types.
            {% endblocktrans %}
        </p>
    {% endif %}

    {% if is_multiple_choice %}
        <form action="{% url 'wagtailadmin_choose_page_chosen_multiple' %}" method="GET" data-multiple-choice-form>

            <div class="page-results">
                {% include 'wagtailadmin/chooser/_browse_results.html' %}
            </div>

            <input type="submit" data-multiple-choice-submit value="{% trans 'Confirm selection' %}" class="button" />
        </form>
    {% else %}
        <div class="page-results">
            {% include 'wagtailadmin/chooser/_browse_results.html' %}
        </div>
    {% endif %}
</div>
