(()=>{class e extends window.wagtailStreamField.blocks.StructBlockDefinition{render(e,t,a,l){const d=super.render(e,t,a,l),i=document.getElementById(`${t}-alt_text`),n=document.getElementById(`${t}-decorative`),c=()=>{n.checked?i.setAttribute("disabled",!0):i.removeAttribute("disabled")};c(),n.addEventListener("change",c);const o=d.childBlocks.image.widget;let r=a?.image?.default_alt_text||"";return o.on("chosen",(e=>{i.value===r&&(i.value=e.default_alt_text),r=e.default_alt_text})),d}}window.telepath.register("wagtail.images.blocks.ImageBlock",e)})();